@@ -1,6 +1,6 @@
 ;;; desktop.el --- save partial status of Emacs when killed
 
-;; Copyright (C) 1993, 1994, 1995, 1997 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
 ;; Author: Morten Welinder <terra@diku.dk>
 ;; Keywords: customization
@@ -35,9 +35,10 @@
 ;;		- buffer-read-only
 ;;		- some local variables
 
-;; To use this, first put these two lines in the bottom of your .emacs
+;; To use this, first put these three lines in the bottom of your .emacs
 ;; file (the later the better):
 ;;
+;;	(load "desktop")
 ;;	(desktop-load-default)
 ;;	(desktop-read)
 ;;
@@ -108,19 +109,9 @@
   "Save status of Emacs when you exit."
   :group 'frames)
 
-(defcustom desktop-enable nil
-  "*Non-nil enable Desktop to save the state of Emacs when you exit."
-  :group 'desktop
-  :type 'boolean
-  :require 'desktop
-  :initialize 'custom-initialize-default
-  :version "20.3")
-
-(defcustom desktop-basefilename
+(defconst desktop-basefilename
   (convert-standard-filename ".emacs.desktop")
-  "File for Emacs desktop, not including the directory name."
-  :type 'file
-  :group 'desktop)
+  "File for Emacs desktop, not including the directory name.")
 
 (defcustom desktop-missing-file-warning nil
   "*If non-nil then desktop warns when a file no longer exists.
@@ -213,11 +204,10 @@ If the function returns t then the buffer is considered created."
   "Opening of form for creation of new buffers.")
 
 (defcustom desktop-save-hook nil
-  "Hook run before desktop saves the state of Emacs.
-This is useful for truncating history lists, for example."
+  "Hook run before saving the desktop to allow you to cut history lists and
+the like shorter."
   :type 'hook
   :group 'desktop)
-
 ;; ----------------------------------------------------------------------------
 (defvar desktop-dirname nil
   "The directory in which the current desktop file resides.")
@@ -230,7 +220,6 @@ This is useful for truncating history lists, for example."
 
 (defvar desktop-delay-hook nil
   "Hooks run after all buffers are loaded; intended for internal use.")
-
 ;; ----------------------------------------------------------------------------
 (defun desktop-truncate (l n)
   "Truncate LIST to at most N elements destructively."
@@ -258,7 +247,6 @@ and those listed in `desktop-clear-preserve-buffers'."
   (let ((buffers (buffer-list)))
     (while buffers
       (or (member (buffer-name (car buffers)) desktop-clear-preserve-buffers)
-	  (null (buffer-name (car buffers)))
 	  ;; Don't kill buffers made for internal purposes.
 	  (and (not (equal (buffer-name (car buffers)) ""))
 	       (eq (aref (buffer-name (car buffers)) 0) ?\ ))
@@ -503,7 +491,6 @@ MODE is the major mode."
 	(if (file-exists-p filename)
 	    (delete-file filename)))))
 ;; ----------------------------------------------------------------------------
-;;;###autoload
 (defun desktop-read ()
   "Read the Desktop file and the files it specifies.
 This is a no-op when Emacs is running in batch mode."
@@ -526,7 +513,6 @@ This is a no-op when Emacs is running in batch mode."
 	    (message "Desktop loaded."))
 	(desktop-clear)))))
 ;; ----------------------------------------------------------------------------
-;;;###autoload
 (defun desktop-load-default ()
   "Load the `default' start-up library manually.
 Also inhibit further loading of it.  Call this from your `.emacs' file
@@ -543,15 +529,12 @@ to provide correct modes for autoloaded files."
       (progn
 	(require 'info)
 	(Info-find-node (nth 0 desktop-buffer-misc) (nth 1 desktop-buffer-misc))
-	(current-buffer))))
+	t)))
 ;; ----------------------------------------------------------------------------
 (defun desktop-buffer-rmail () "Load an RMAIL file."
   (if (eq 'rmail-mode desktop-buffer-major-mode)
       (condition-case error
-	  (progn (rmail-input desktop-buffer-file-name)
-                         (if (eq major-mode 'rmail-mode)
-                             (current-buffer)
-                           rmail-buffer))
+	  (progn (rmail-input desktop-buffer-file-name) t)
 	(file-locked
 	 (kill-buffer (current-buffer))
 	 'ignored))))
@@ -561,16 +544,16 @@ to provide correct modes for autoloaded files."
       (progn
 	(require 'mh-e)
 	(mh-find-path)
-        (mh-visit-folder desktop-buffer-name)
-	(current-buffer))))
+	(mh-visit-folder desktop-buffer-name)
+	t)))
 ;; ----------------------------------------------------------------------------
 (defun desktop-buffer-dired () "Load a directory using dired."
   (if (eq 'dired-mode desktop-buffer-major-mode)
       (if (file-directory-p (file-name-directory (car desktop-buffer-misc)))
 	  (progn
-            (dired (car desktop-buffer-misc))
-	    (mapcar 'dired-maybe-insert-subdir (cdr desktop-buffer-misc))
-	    (current-buffer))
+	    (dired (car desktop-buffer-misc))
+	    (mapcar 'dired-insert-subdir (cdr desktop-buffer-misc))
+	    t)
 	(message "Directory %s no longer exists." (car desktop-buffer-misc))
 	(sit-for 1)
 	'ignored)))
@@ -582,7 +565,7 @@ to provide correct modes for autoloaded files."
 		   (y-or-n-p (format
 			      "File \"%s\" no longer exists. Re-create? "
 			      desktop-buffer-file-name))))
-	  (progn (find-file desktop-buffer-file-name) (current-buffer))
+	  (progn (find-file desktop-buffer-file-name) t)
 	'ignored)))
 ;; ----------------------------------------------------------------------------
 ;; Create a buffer, load its file, set is mode, ...;  called from Desktop file
@@ -597,30 +580,31 @@ to provide correct modes for autoloaded files."
       (setq handler (car hlist))
       (setq result (funcall handler))
       (setq hlist (cdr hlist)))
-    (when (bufferp result)
-      (set-buffer result)
-      (if (not (equal (buffer-name) desktop-buffer-name))
-	  (rename-buffer desktop-buffer-name))
-      (auto-fill-mode (if (nth 0 mim) 1 0))
-      (goto-char pt)
-      (if (consp mk)
-	  (progn
-	    (set-mark (car mk))
-	    (setq mark-active (car (cdr mk))))
-	(set-mark mk))
-      ;; Never override file system if the file really is read-only marked.
-      (if ro (setq buffer-read-only ro))
-      (while locals
-	(let ((this (car locals)))
-	  (if (consp this)
-	      ;; an entry of this form `(symbol . value)'
+    (if (eq result t)
+	(progn
+	  (if (not (equal (buffer-name) desktop-buffer-name))
+	      (rename-buffer desktop-buffer-name))
+	  (auto-fill-mode (if (nth 0 mim) 1 0))
+	  (goto-char pt)
+	  (if (consp mk)
 	      (progn
-		(make-local-variable (car this))
-		(set (car this) (cdr this)))
-	    ;; an entry of the form `symbol'
-	    (make-local-variable this)
-	    (makunbound this)))
-	(setq locals (cdr locals))))))
+		(set-mark (car mk))
+		(setq mark-active (car (cdr mk))))
+	    (set-mark mk))
+	  ;; Never override file system if the file really is read-only marked.
+	  (if ro (setq buffer-read-only ro))
+	  (while locals
+	    (let ((this (car locals)))
+	      (if (consp this)
+		  ;; an entry of this form `(symbol . value)'
+		  (progn
+		    (make-local-variable (car this))
+		    (set (car this) (cdr this)))
+		;; an entry of the form `symbol'
+		(make-local-variable this)
+		(makunbound this)))
+	    (setq locals (cdr locals)))
+	  ))))
 
 ;; Backward compatibility -- update parameters to 205 standards.
 (defun desktop-buffer (desktop-buffer-file-name desktop-buffer-name
@@ -635,16 +619,6 @@ to provide correct modes for autoloaded files."
 			       (cons 'case-replace cr)
 			       (cons 'overwrite-mode (car mim)))))
 ;; ----------------------------------------------------------------------------
-
-;; If the user set desktop-enable to t with Custom,
-;; do the rest of what it takes to use desktop,
-;; but do it after finishing loading the init file.
-(add-hook 'after-init-hook
-	  '(lambda ()
-	     (when desktop-enable
-	       (desktop-load-default)
-	       (desktop-read))))
-
 (provide 'desktop)
 
 ;; desktop.el ends here.
