@@ -17,8 +17,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -68,14 +69,19 @@ Calls value of `electric-buffer-menu-mode-hook' on entry if non-nil.
 			    ?\ )
 			(progn (setq unread-command-events nil)
 			       (throw 'electric-buffer-menu-select nil)))
-		    (let ((first (progn (goto-char (point-min))
+		    (let ((start-point (point))
+			  (first (progn (goto-char (point-min))
 					(forward-line 2)
 					(point)))
 			  (last (progn (goto-char (point-max))
 				       (forward-line -1)
 				       (point)))
 			  (goal-column 0))
-		      (goto-char first)
+		      ;; Use start-point if it is meaningful.
+		      (goto-char (if (or (< start-point first)
+					 (> start-point last))
+				     first
+				   start-point))
 		      (Electric-command-loop 'electric-buffer-menu-select
 					     nil
 					     t
@@ -233,7 +239,8 @@ Does not execute select, save, or delete commands."
 (defun Electric-buffer-menu-undefined ()
   (interactive)
   (ding)
-  (message (if (and (eq (key-binding "\C-c\C-c") 'Electric-buffer-menu-quit)
+  (message "%s"
+	   (if (and (eq (key-binding "\C-c\C-c") 'Electric-buffer-menu-quit)
 		    (eq (key-binding " ") 'Electric-buffer-menu-select)
 		    (eq (key-binding (char-to-string help-char)) 'Helper-help)
 		    (eq (key-binding "?") 'Helper-describe-bindings))
@@ -255,4 +262,6 @@ Returns to Electric Buffer Menu when done."
       (message "Buffer %s does not exist!" bufnam)
       (sit-for 4))))
 
+(provide 'ebuff-menu)
+
 ;;; ebuff-menu.el ends here
