@@ -1,8 +1,9 @@
 ;;; server.el --- Lisp code for GNU Emacs running as server process.
 
-;; Copyright (C) 1986, 1987, 1992, 1994, 1995 Free Software Foundation, Inc.
+;; Copyright (C) 1986, 87, 92, 94, 95, 96, 1997 Free Software Foundation, Inc.
 
 ;; Author: William Sommerfeld <wesommer@athena.mit.edu>
+;; Maintainer: FSF
 ;; Keywords: processes
 
 ;; Changes by peck@sun.com and by rms.
@@ -20,70 +21,83 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
-;;; This Lisp code is run in Emacs when it is to operate as
-;;; a server for other processes.
+;; This Lisp code is run in Emacs when it is to operate as
+;; a server for other processes.
 
-;;; Load this library and do M-x server-edit to enable Emacs as a server.
-;;; Emacs runs the program ../arch-lib/emacsserver as a subprocess
-;;; for communication with clients.  If there are no client buffers to edit, 
-;;; server-edit acts like (switch-to-buffer (other-buffer))
+;; Load this library and do M-x server-edit to enable Emacs as a server.
+;; Emacs runs the program ../arch-lib/emacsserver as a subprocess
+;; for communication with clients.  If there are no client buffers to edit, 
+;; server-edit acts like (switch-to-buffer (other-buffer))
 
-;;; When some other program runs "the editor" to edit a file,
-;;; "the editor" can be the Emacs client program ../lib-src/emacsclient.
-;;; This program transmits the file names to Emacs through
-;;; the server subprocess, and Emacs visits them and lets you edit them.
+;; When some other program runs "the editor" to edit a file,
+;; "the editor" can be the Emacs client program ../lib-src/emacsclient.
+;; This program transmits the file names to Emacs through
+;; the server subprocess, and Emacs visits them and lets you edit them.
 
-;;; Note that any number of clients may dispatch files to emacs to be edited.
+;; Note that any number of clients may dispatch files to emacs to be edited.
 
-;;; When you finish editing a Server buffer, again call server-edit
-;;; to mark that buffer as done for the client and switch to the next 
-;;; Server buffer.  When all the buffers for a client have been edited 
-;;; and exited with server-edit, the client "editor" will return
-;;; to the program that invoked it.  
+;; When you finish editing a Server buffer, again call server-edit
+;; to mark that buffer as done for the client and switch to the next 
+;; Server buffer.  When all the buffers for a client have been edited 
+;; and exited with server-edit, the client "editor" will return
+;; to the program that invoked it.  
 
-;;; Your editing commands and Emacs's display output go to and from
-;;; the terminal in the usual way.  Thus, server operation is possible
-;;; only when Emacs can talk to the terminal at the time you invoke
-;;; the client.  This is possible in four cases:
+;; Your editing commands and Emacs's display output go to and from
+;; the terminal in the usual way.  Thus, server operation is possible
+;; only when Emacs can talk to the terminal at the time you invoke
+;; the client.  This is possible in four cases:
 
-;;; 1. On a window system, where Emacs runs in one window and the
-;;; program that wants to use "the editor" runs in another.
+;; 1. On a window system, where Emacs runs in one window and the
+;; program that wants to use "the editor" runs in another.
 
-;;; 2. On a multi-terminal system, where Emacs runs on one terminal and the
-;;; program that wants to use "the editor" runs on another.
+;; 2. On a multi-terminal system, where Emacs runs on one terminal and the
+;; program that wants to use "the editor" runs on another.
 
-;;; 3. When the program that wants to use "the editor" is running
-;;; as a subprocess of Emacs.
+;; 3. When the program that wants to use "the editor" is running
+;; as a subprocess of Emacs.
 
-;;; 4. On a system with job control, when Emacs is suspended, the program
-;;; that wants to use "the editor" will stop and display
-;;; "Waiting for Emacs...".  It can then be suspended, and Emacs can be
-;;; brought into the foreground for editing.  When done editing, Emacs is
-;;; suspended again, and the client program is brought into the foreground.
+;; 4. On a system with job control, when Emacs is suspended, the program
+;; that wants to use "the editor" will stop and display
+;; "Waiting for Emacs...".  It can then be suspended, and Emacs can be
+;; brought into the foreground for editing.  When done editing, Emacs is
+;; suspended again, and the client program is brought into the foreground.
 
-;;; The buffer local variable "server-buffer-clients" lists 
-;;; the clients who are waiting for this buffer to be edited.  
-;;; The global variable "server-clients" lists all the waiting clients,
-;;; and which files are yet to be edited for each.
+;; The buffer local variable "server-buffer-clients" lists 
+;; the clients who are waiting for this buffer to be edited.  
+;; The global variable "server-clients" lists all the waiting clients,
+;; and which files are yet to be edited for each.
 
 ;;; Code:
 
-(defvar server-program (expand-file-name "emacsserver" exec-directory)
-  "*The program to use as the edit server.")
+(defgroup server nil
+  "Emacs running as a server process."
+  :group 'external)
 
-(defvar server-visit-hook nil
-  "*List of hooks to call when visiting a file for the Emacs server.")
+(defcustom server-program (expand-file-name "emacsserver" exec-directory)
+  "*The program to use as the edit server."
+  :group 'server
+  :type 'string)
 
-(defvar server-switch-hook nil
-  "*List of hooks to call when switching to a buffer for the Emacs server.")
+(defcustom server-visit-hook nil
+  "*List of hooks to call when visiting a file for the Emacs server."
+  :group 'server
+  :type '(repeat function))
 
-(defvar server-done-hook nil
-  "*List of hooks to call when done editing a buffer for the Emacs server.")
+(defcustom server-switch-hook nil
+  "*List of hooks to call when switching to a buffer for the Emacs server."
+  :group 'server
+  :type '(repeat function))
+
+(defcustom server-done-hook nil
+  "*List of hooks to call when done editing a buffer for the Emacs server."
+  :group 'server
+  :type '(repeat function))
 
 (defvar server-process nil 
   "the current server process")
@@ -107,10 +121,12 @@ When a buffer is marked as \"done\", it is removed from this list.")
 If nil, use the selected window.
 If it is a frame, use the frame's selected window.")
 
-(defvar server-temp-file-regexp "^/tmp/Re\\|/draft$"
+(defcustom server-temp-file-regexp "^/tmp/Re\\|/draft$"
   "*Regexp which should match filenames of temporary files
 which are deleted and reused after each edit
-by the programs that invoke the emacs server.")
+by the programs that invoke the emacs server."
+  :group 'server
+  :type 'regexp)
 
 (or (assq 'server-buffer-clients minor-mode-alist)
     (setq minor-mode-alist (cons '(server-buffer-clients " Server") minor-mode-alist)))
@@ -122,8 +138,8 @@ by the programs that invoke the emacs server.")
       (save-excursion
 	(set-buffer "*server*")
 	(goto-char (point-max))
-	(insert string)
-	(or (bobp) (newline)))))
+	(insert (current-time-string) " " string)
+	(or (bolp) (newline)))))
 
 (defun server-sentinel (proc msg)
   (cond ((eq (process-status proc) 'exit)
@@ -146,11 +162,26 @@ Prefix arg means just kill any existing server communications subprocess."
       (progn
 	(set-process-sentinel server-process nil)
 	(condition-case () (delete-process server-process) (error nil))))
-  (condition-case () (delete-file "~/.emacs_server") (error nil))
-  (condition-case ()
-      (delete-file (format "/tmp/esrv%d-%s" (user-uid) (system-name)))
-    (error nil))
-  ;; If we already had a server, clear out associated status.
+  ;; Delete the socket files made by previous server invocations.
+  (let* ((sysname (system-name))
+	 (dot-index (string-match "\\." sysname)))
+    (condition-case ()
+	(delete-file (format "~/.emacs-server-%s" sysname))
+      (error nil))
+    (condition-case ()
+	(delete-file (format "/tmp/esrv%d-%s" (user-uid) sysname))
+      (error nil))
+    ;; In case the server file name was made with a domainless hostname,
+    ;; try deleting that name too.
+    (if dot-index
+	(let ((shortname (substring sysname 0 dot-index)))
+	  (condition-case ()
+	      (delete-file (format "~/.emacs-server-%s" shortname))
+	    (error nil))
+	  (condition-case ()
+	      (delete-file (format "/tmp/esrv%d-%s" (user-uid) shortname))
+	    (error nil)))))
+  ;; If this Emacs already had a server, clear out associated status.
   (while server-clients
     (let ((buffer (nth 1 (car server-clients))))
       (server-buffer-done buffer)))
@@ -175,38 +206,62 @@ Prefix arg means just kill any existing server communications subprocess."
   ;; process each line individually.
   (while (string-match "\n" string)
     (let ((request (substring string 0 (match-beginning 0)))
-	  client
+	  client nowait
 	  (files nil)
 	  (lineno 1))
       ;; Remove this line from STRING.
       (setq string (substring string (match-end 0)))	  
-      (if (string-match "^Client: " request)
-	  (setq request (substring request (match-end 0))))
-      (setq client (list (substring request 0 (string-match " " request))))
-      (setq request (substring request (match-end 0)))
-      (while (string-match "[^ ]+ " request)
-	(let ((arg
-	       (substring request (match-beginning 0) (1- (match-end 0)))))
-	  (setq request (substring request (match-end 0)))
-	  (if (string-match "\\`\\+[0-9]+\\'" arg)
-	      (setq lineno (read (substring arg 1)))
-	    (setq files
-		  (cons (list arg lineno)
-			files))
-	    (setq lineno 1))))
-      (server-visit-files files client)
-      ;; CLIENT is now a list (CLIENTNUM BUFFERS...)
-      (setq server-clients (cons client server-clients))
-      (server-switch-buffer (nth 1 client))
-      (run-hooks 'server-switch-hook)
-      (message (substitute-command-keys
-		"When done with a buffer, type \\[server-edit]."))))
+      (if (string-match "^Error: " request)
+	  (message "Server error: %s" (substring request (match-end 0)))
+	(if (string-match "^Client: " request)
+	    (progn
+	      (setq request (substring request (match-end 0)))
+	      (setq client (list (substring request 0 (string-match " " request))))
+	      (setq request (substring request (match-end 0)))
+	      (while (string-match "[^ ]+ " request)
+		(let ((arg
+		       (substring request (match-beginning 0) (1- (match-end 0))))
+		      (pos 0))
+		  (setq request (substring request (match-end 0)))
+		  (if (string-match "\\`-nowait" arg)
+		      (setq nowait t)
+		    (if (string-match "\\`\\+[0-9]+\\'" arg)
+			;; ARG is a line number option.
+			(setq lineno (read (substring arg 1)))
+		      ;; ARG is a file name.
+		      ;; Collapse multiple slashes to single slashes.
+		      (setq arg (command-line-normalize-file-name arg))
+		      ;; Undo the quoting that emacsclient does
+		      ;; for certain special characters.
+		      (while (string-match "&." arg pos)
+			(setq pos (1+ (match-beginning 0)))
+			(let ((nextchar (aref arg pos)))
+			  (cond ((= nextchar ?&)
+				 (setq arg (replace-match "&" t t arg)))
+				((= nextchar ?-)
+				 (setq arg (replace-match "-" t t arg)))
+				(t
+				 (setq arg (replace-match " " t t arg))))))
+		      (setq files
+			    (cons (list arg lineno)
+				  files))
+		      (setq lineno 1)))))
+	      (server-visit-files files client nowait)
+	      ;; CLIENT is now a list (CLIENTNUM BUFFERS...)
+	      (or nowait
+		  (setq server-clients (cons client server-clients)))
+	      (server-switch-buffer (nth 1 client))
+	      (run-hooks 'server-switch-hook)
+	      (message (substitute-command-keys
+			"When done with a buffer, type \\[server-edit]")))))))
   ;; Save for later any partial line that remains.
   (setq server-previous-string string))
 
-(defun server-visit-files (files client)
+(defun server-visit-files (files client &optional nowait)
   "Finds FILES and returns the list CLIENT with the buffers nconc'd.
-FILES is an alist whose elements are (FILENAME LINENUMBER)."
+FILES is an alist whose elements are (FILENAME LINENUMBER).
+NOWAIT non-nil means this client is not waiting for the results,
+so don't mark these buffers specially, just visit them normally."
   ;; Bind last-nonmenu-event to force use of keyboard, not mouse, for queries.
   (let (client-record (last-nonmenu-event t) (obuf (current-buffer)))
     ;; Restore the current buffer afterward, but not using save-excursion,
@@ -232,13 +287,15 @@ FILES is an alist whose elements are (FILENAME LINENUMBER)."
 	      (set-buffer (find-file-noselect filen))
 	      (run-hooks 'server-visit-hook)))
 	  (goto-line (nth 1 (car files)))
-	  (setq server-buffer-clients (cons (car client) server-buffer-clients))
+	  (if (not nowait)
+	      (setq server-buffer-clients
+		    (cons (car client) server-buffer-clients)))
 	  (setq client-record (cons (current-buffer) client-record))
 	  (setq files (cdr files)))
       (set-buffer obuf))
     (nconc client client-record)))
 
-(defun server-buffer-done (buffer)
+(defun server-buffer-done (buffer &optional for-killing)
   "Mark BUFFER as \"done\" for its client(s).
 This buries the buffer, then returns a list of the form (NEXT-BUFFER KILLED).
 NEXT-BUFFER is another server buffer, as a suggestion for what to select next,
@@ -246,6 +303,7 @@ or nil.  KILLED is t if we killed BUFFER (because it was a temp file)."
   (let ((running (eq (process-status server-process) 'run))
 	(next-buffer nil)
 	(killed nil)
+	(first t)
 	(old-clients server-clients))
     (while old-clients
       (let ((client (car old-clients)))
@@ -264,12 +322,13 @@ or nil.  KILLED is t if we killed BUFFER (because it was a temp file)."
 	(if (cdr client) nil
 	  (if running
 	      (progn
-		(send-string server-process 
-			     (format "Close: %s Done\n" (car client)))
-		(server-log (format "Close: %s Done\n" (car client)))
 		;; Don't send emacsserver two commands in close succession.
 		;; It cannot handle that.
-		(sit-for 1)))
+		(or first (sit-for 1))
+		(setq first nil)
+		(send-string server-process 
+			     (format "Close: %s Done\n" (car client)))
+		(server-log (format "Close: %s Done\n" (car client)))))
 	  (setq server-clients (delq client server-clients))))
       (setq old-clients (cdr old-clients)))
     (if (and (bufferp buffer) (buffer-name buffer))
@@ -278,10 +337,11 @@ or nil.  KILLED is t if we killed BUFFER (because it was a temp file)."
 	    (set-buffer buffer)
 	    (setq server-buffer-clients nil)
 	    (run-hooks 'server-done-hook))
-	  (if (server-temp-file-p buffer)
-	      (progn (kill-buffer buffer)
-		     (setq killed t))
-	    (bury-buffer buffer))))
+	  (if for-killing
+	      (if (server-temp-file-p buffer)
+		  (progn (kill-buffer buffer)
+			 (setq killed t))
+		(bury-buffer buffer)))))
     (list next-buffer killed)))
 
 (defun server-temp-file-p (buffer)
@@ -309,6 +369,7 @@ or nil.  KILLED is t if we killed the BUFFER (because it was a temp file)."
 		    (buffer-backed-up nil))
 		(save-buffer))
 	    (if (and (buffer-modified-p)
+		     buffer-file-name
 		     (y-or-n-p (concat "Save file " buffer-file-name "? ")))
 		(save-buffer buffer)))
 	  (server-buffer-done buffer)))))
@@ -337,6 +398,19 @@ or nil.  KILLED is t if we killed the BUFFER (because it was a temp file)."
 	(yes-or-no-p "Server buffers still have clients; exit anyway? "))))
 
 (add-hook 'kill-emacs-query-functions 'server-kill-emacs-query-function)
+
+(defvar server-kill-buffer-running nil
+  "Non-nil while `server-kill-buffer' is running.")
+
+;; When a buffer is killed, inform the clients.
+(add-hook 'kill-buffer-hook 'server-kill-buffer)
+(defun server-kill-buffer ()
+  ;; Prevent infinite recursion if user has made server-done-hook
+  ;; call kill-buffer.
+  (or server-kill-buffer-running
+      (let ((server-kill-buffer-running t))
+	(when server-process
+	  (server-buffer-done (current-buffer) t)))))
 
 (defun server-edit (&optional arg)
   "Switch to next server editing buffer; say \"Done\" for current buffer.
