@@ -90,7 +90,7 @@ fail (char * msg, ...)
   vfprintf (stderr, msg, args);
   va_end (args);
 
-  exit (-1);
+  exit (1);
 }
 
 void
@@ -366,18 +366,12 @@ console_event_handler (DWORD event)
   return TRUE;
 }
 
-/* Change from normal usage; return value indicates whether spawn
-   succeeded or failed - program return code is returned separately.  */
 int
-spawn (char * progname, char * cmdline, int * retcode)
+spawn (char * progname, char * cmdline)
 {
-  BOOL success = FALSE;
+  DWORD rc = 0xff;
   SECURITY_ATTRIBUTES sec_attrs;
   STARTUPINFO start;
-  /* In theory, passing NULL for the environment block to CreateProcess
-     is the same as passing the value of GetEnvironmentStrings, but
-     doing this explicitly seems to cure problems running DOS programs
-     in some cases.  */
   char * envblock = GetEnvironmentStrings ();
 
   sec_attrs.nLength = sizeof (sec_attrs);
@@ -390,11 +384,9 @@ spawn (char * progname, char * cmdline, int * retcode)
   if (CreateProcess (progname, cmdline, &sec_attrs, NULL, TRUE,
 		     0, envblock, NULL, &start, &child))
   {
-    success = TRUE;
     /* wait for completion and pass on return code */
     WaitForSingleObject (child.hProcess, INFINITE);
-    if (retcode)
-      GetExitCodeProcess (child.hProcess, (DWORD *)retcode);
+    GetExitCodeProcess (child.hProcess, &rc);
     CloseHandle (child.hThread);
     CloseHandle (child.hProcess);
     child.hProcess = NULL;
@@ -402,7 +394,7 @@ spawn (char * progname, char * cmdline, int * retcode)
 
   FreeEnvironmentStrings (envblock);
 
-  return success;
+  return (int) rc;
 }
 
 /* Return size of current environment block.  */
@@ -462,9 +454,7 @@ main (int argc, char ** argv)
       /* We are being used as a helper to run a DOS app; just pass
 	 command line to DOS app without change.  */
       /* TODO: fill in progname.  */
-      if (spawn (NULL, GetCommandLine (), &rc))
-	return rc;
-      fail ("Could not run %s\n", GetCommandLine ());
+      return spawn (NULL, GetCommandLine ());
     }
 
   /* Process command line.  If running interactively (-c or /c not
@@ -543,7 +533,7 @@ main (int argc, char ** argv)
 #else
   /* Probably a mistake for there to be extra args; not fatal.  */
   if (argc > 0)
-    warn ("warning: extra args ignored after '%s'\n", argv[-1]);
+    warn ("warning: extra args ignored after %s\n", argv[-1]);
 #endif
 
   pass_through_args[num_pass_through_args] = NULL;
@@ -578,7 +568,6 @@ main (int argc, char ** argv)
 	}
     }
 
- pass_to_shell:
   if (need_shell)
     {
       char * p;
@@ -620,32 +609,22 @@ main (int argc, char ** argv)
 	  for (argv = pass_through_args; *argv != NULL; ++argv)
 	    p += wsprintf (p, " %s", *argv);
 
-	  if (GetVersion () & 0x80000000)
-	    /* Set environment size to something reasonable on Windows 95.  */
-	    wsprintf(p, " /e:%d /c %s", envsize, cmdline);
-	  else
-	    wsprintf(p, " /c %s", cmdline);
+	  /* Always set environment size to something reasonable.  */
+	  wsprintf(p, " /e:%d /c %s", envsize, cmdline);
 	  cmdline = buf;
 	}
       else
 	{
-	  if (GetVersion () & 0x80000000)
-	    {
-	      /* Provide dir arg expected by command.com when first
-		 started interactively (the "command search path").
-		 cmd.exe does not require it, but accepts it silently -
-		 presumably other DOS compatible shells do the same.  To
-		 avoid potential problems with spaces in command dir
-		 (which cannot be quoted - command.com doesn't like it),
-		 we always use the 8.3 form.  */
-	      GetShortPathName (progname, path, sizeof (path));
-	      p = strrchr (path, '\\');
-	      /* Trailing slash is acceptable, so always leave it.  */
-	      *(++p) = '\0';
-	    }
-	  else
-	    /* Dir arg not needed on NT.  */
-	    path[0] = '\0';
+	  /* Provide dir arg expected by command.com when first started
+	     interactively (the "command search path").  cmd.exe does
+	     not require it, but accepts it silently - presumably other
+	     DOS compatible shells do the same.  To avoid potential
+	     problems with spaces in command dir (which cannot be quoted
+	     - command.com doesn't like it), we always use the 8.3 form.  */
+	  GetShortPathName (progname, path, sizeof (path));
+	  p = strrchr (path, '\\');
+	  /* Trailing slash is acceptable, so always leave it.  */
+	  *(++p) = '\0';
 
 	  cmdline = p = alloca (strlen (progname) + extra_arg_space +
 				strlen (path) + 13);
@@ -658,9 +637,9 @@ main (int argc, char ** argv)
 	  for (argv = pass_through_args; *argv != NULL; ++argv)
 	    p += wsprintf (p, " %s", *argv);
 
-	  if (GetVersion () & 0x80000000)
-	    /* Set environment size to something reasonable on Windows 95.  */
-	    wsprintf (p, " /e:%d", envsize);
+	  /* Always set environment size to something reasonable - again
+             cmd.exe ignores this silently.  */
+	  wsprintf (p, " /e:%d", envsize);
 	}
     }
 
@@ -670,16 +649,7 @@ main (int argc, char ** argv)
   if (!cmdline)
     cmdline = progname;
 
-  if (spawn (progname, cmdline, &rc))
-    return rc;
+  rc = spawn (progname, cmdline);
 
-  if (!need_shell)
-    {
-      need_shell = TRUE;
-      goto pass_to_shell;
-    }
-
-  fail ("Could not run %s\n", progname);
-
-  return 0;
+  return rc;
 }
