@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,16 +15,20 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 
 #include <config.h>
 #include "lisp.h"
 #include "buffer.h"
+#include "charset.h"
 #include "commands.h"
 #include "syntax.h"
 
 enum case_action {CASE_UP, CASE_DOWN, CASE_CAPITALIZE, CASE_CAPITALIZE_UP};
+
+Lisp_Object Qidentity;
 
 Lisp_Object
 casify_object (flag, obj)
@@ -33,36 +37,79 @@ casify_object (flag, obj)
 {
   register int i, c, len;
   register int inword = flag == CASE_DOWN;
+  Lisp_Object tem;
+
+  /* If the case table is flagged as modified, rescan it.  */
+  if (NILP (XCHAR_TABLE (current_buffer->downcase_table)->extras[1]))
+    Fset_case_table (current_buffer->downcase_table);
 
   while (1)
     {
-      if (XTYPE (obj) == Lisp_Int)
+      if (INTEGERP (obj))
 	{
-	  c = XINT (obj);
-	  if (c >= 0 && c <= 0400)
+	  c = DOWNCASE (XFASTINT (obj));
+	  if (inword)
+	    XSETFASTINT (obj, c);
+	  else if (c == XFASTINT (obj))
 	    {
-	      if (inword)
-		XFASTINT (obj) = DOWNCASE (c);
-	      else if (!UPPERCASEP (c))
-		XFASTINT (obj) = UPCASE1 (c);
+	      c = UPCASE1 (XFASTINT (obj));
+	      XSETFASTINT (obj, c);
 	    }
 	  return obj;
 	}
-      if (XTYPE (obj) == Lisp_String)
+      if (STRINGP (obj))
 	{
+	  int multibyte = !NILP (current_buffer->enable_multibyte_characters);
+
 	  obj = Fcopy_sequence (obj);
 	  len = XSTRING (obj)->size;
 	  for (i = 0; i < len; i++)
 	    {
 	      c = XSTRING (obj)->data[i];
-	      if (inword)
+	      if (multibyte && c >= 0x80)
+		/* A multibyte character can't be handled in this
+                   simple loop.  */
+		break;
+	      if (inword && flag != CASE_CAPITALIZE_UP)
 		c = DOWNCASE (c);
-	      else if (!UPPERCASEP (c))
+	      else if (!UPPERCASEP (c)
+		       && (!inword || flag != CASE_CAPITALIZE_UP))
 		c = UPCASE1 (c);
 	      XSTRING (obj)->data[i] = c;
-	      if (flag == CASE_CAPITALIZE)
+	      if ((int) flag >= (int) CASE_CAPITALIZE)
 		inword = SYNTAX (c) == Sword;
 	    }
+	  if (i < len)
+	    {
+	      /* The work is not yet finished because of a multibyte
+		 character just encountered.  */
+	      int fromlen, tolen, j = i;
+	      char *buf
+		= (char *) alloca ((len - i) * MAX_LENGTH_OF_MULTI_BYTE_FORM
+				   + i);
+	      char *str, workbuf[4];
+
+	      /* Copy data already handled.  */
+	      bcopy (XSTRING (obj)->data, buf, i);
+
+	      while (i < len)
+		{
+		  c = STRING_CHAR_AND_LENGTH (XSTRING (obj)->data + i,
+					      len - i, fromlen);
+		  if (inword && flag != CASE_CAPITALIZE_UP)
+		    c = DOWNCASE (c);
+		  else if (!UPPERCASEP (c)
+			   && (!inword || flag != CASE_CAPITALIZE_UP))
+		    c = UPCASE1 (c);
+		  tolen = CHAR_STRING (c, workbuf, str);
+		  bcopy (str, buf + j, tolen);
+		  i += fromlen;
+		  j += tolen;
+		  if ((int) flag >= (int) CASE_CAPITALIZE)
+		    inword = SYNTAX (c) == Sword;
+		}
+	      obj = make_string (buf, j);
+	    }
 	  return obj;
 	}
       obj = wrong_type_argument (Qchar_or_string_p, obj);
@@ -72,7 +119,8 @@ casify_object (flag, obj)
 DEFUN ("upcase", Fupcase, Supcase, 1, 1, 0,
   "Convert argument to upper case and return that.\n\
 The argument may be a character or string.  The result has the same type.\n\
-The argument object is not altered.  See also `capitalize'.")
+The argument object is not altered--the value is a copy.\n\
+See also `capitalize', `downcase' and `upcase-initials'.")
   (obj)
      Lisp_Object obj;
 {
@@ -82,7 +130,7 @@ The argument object is not altered.  See also `capitalize'.")
 DEFUN ("downcase", Fdowncase, Sdowncase, 1, 1, 0,
   "Convert argument to lower case and return that.\n\
 The argument may be a character or string.  The result has the same type.\n\
-The argument object is not altered.")
+The argument object is not altered--the value is a copy.")
   (obj)
      Lisp_Object obj;
 {
@@ -94,12 +142,25 @@ DEFUN ("capitalize", Fcapitalize, Scapitalize, 1, 1, 0,
 This means that each word's first character is upper case\n\
 and the rest is lower case.\n\
 The argument may be a character or string.  The result has the same type.\n\
-The argument object is not altered.")
+The argument object is not altered--the value is a copy.")
   (obj)
      Lisp_Object obj;
 {
   return casify_object (CASE_CAPITALIZE, obj);
 }
+
+/* Like Fcapitalize but change only the initials.  */
+
+DEFUN ("upcase-initials", Fupcase_initials, Supcase_initials, 1, 1, 0,
+  "Convert the initial of each word in the argument to upper case.\n\
+Do not change the other letters of each word.\n\
+The argument may be a character or string.  The result has the same type.\n\
+The argument object is not altered--the value is a copy.")
+  (obj)
+     Lisp_Object obj;
+{
+  return casify_object (CASE_CAPITALIZE_UP, obj);
+}
 
 /* flag is CASE_UP, CASE_DOWN or CASE_CAPITALIZE or CASE_CAPITALIZE_UP.
    b and e specify range of buffer to operate on. */
@@ -111,31 +172,92 @@ casify_region (flag, b, e)
   register int i;
   register int c;
   register int inword = flag == CASE_DOWN;
+  register int multibyte = !NILP (current_buffer->enable_multibyte_characters);
+  int start, end;
+  Lisp_Object ch, downch, val;
 
   if (EQ (b, e))
     /* Not modifying because nothing marked */
     return;
 
-  validate_region (&b, &e);
-  modify_region (current_buffer, XFASTINT (b), XFASTINT (e));
-  record_change (XFASTINT (b), XFASTINT (e) - XFASTINT (b));
+  /* If the case table is flagged as modified, rescan it.  */
+  if (NILP (XCHAR_TABLE (current_buffer->downcase_table)->extras[1]))
+    Fset_case_table (current_buffer->downcase_table);
 
-  for (i = XFASTINT (b); i < XFASTINT (e); i++)
+  validate_region (&b, &e);
+  start = XFASTINT (b);
+  end = XFASTINT (e);
+  modify_region (current_buffer, start, end);
+  record_change (start, end - start);
+
+  for (i = start; i < end; i++)
     {
-      c = FETCH_CHAR (i);
+      c = FETCH_BYTE (i);
+      if (multibyte && c >= 0x80)
+	/* A multibyte character can't be handled in this simple loop.  */
+	break;
       if (inword && flag != CASE_CAPITALIZE_UP)
 	c = DOWNCASE (c);
       else if (!UPPERCASEP (c)
 	       && (!inword || flag != CASE_CAPITALIZE_UP))
 	c = UPCASE1 (c);
-      FETCH_CHAR (i) = c;
+      FETCH_BYTE (i) = c;
       if ((int) flag >= (int) CASE_CAPITALIZE)
 	inword = SYNTAX (c) == Sword;
     }
+  if (i < end)
+    {
+      /* The work is not yet finished because of a multibyte character
+	 just encountered.  */
+      int opoint = PT, c2;
 
-  signal_after_change (XFASTINT (b),
-		       XFASTINT (e) - XFASTINT (b), 
-		       XFASTINT (e) - XFASTINT (b));
+      while (i < end)
+	{
+	  if ((c = FETCH_BYTE (i)) >= 0x80)
+	    c = FETCH_MULTIBYTE_CHAR (i);
+	  c2 = c;
+	  if (inword && flag != CASE_CAPITALIZE_UP)
+	    c2 = DOWNCASE (c);
+	  else if (!UPPERCASEP (c)
+		   && (!inword || flag != CASE_CAPITALIZE_UP))
+	    c2 = UPCASE1 (c);
+	  if (c != c2)
+	    {
+	      int fromlen, tolen, j;
+	      char workbuf[4], *str;
+
+	      /* Handle the most likely case */
+	      if (c < 0400 && c2 < 0400)
+		FETCH_BYTE (i) = c2;
+	      else if (fromlen = CHAR_STRING (c, workbuf, str),
+		       tolen = CHAR_STRING (c2, workbuf, str),
+		       fromlen == tolen)
+		{
+		  for (j = 0; j < tolen; ++j)
+		    FETCH_BYTE (i + j) = str[j];
+		}
+	      else
+		{
+		  error ("Can't casify letters that change length");
+#if 0 /* This is approximately what we'd like to be able to do here */
+		  if (tolen < fromlen)
+		    del_range_1 (i + tolen, i + fromlen, 0);
+		  else if (tolen > fromlen)
+		    {
+		      TEMP_SET_PT (i + fromlen);
+		      insert_1 (str + fromlen, tolen - fromlen, 1, 0);
+		    }
+#endif
+		}
+	    }
+	  if ((int) flag >= (int) CASE_CAPITALIZE)
+	    inword = SYNTAX (c2) == Sword;
+	  INC_POS (i);
+	}
+      TEMP_SET_PT (opoint);
+    }
+
+  signal_after_change (start, end - start, end - start);
 }
 
 DEFUN ("upcase-region", Fupcase_region, Supcase_region, 2, 2, "r",
@@ -144,10 +266,10 @@ These arguments specify the starting and ending character numbers of\n\
 the region to operate on.  When used as a command, the text between\n\
 point and the mark is operated on.\n\
 See also `capitalize-region'.")
-  (b, e)
-     Lisp_Object b, e;
+  (beg, end)
+     Lisp_Object beg, end;
 {
-  casify_region (CASE_UP, b, e);
+  casify_region (CASE_UP, beg, end);
   return Qnil;
 }
 
@@ -156,10 +278,10 @@ DEFUN ("downcase-region", Fdowncase_region, Sdowncase_region, 2, 2, "r",
 These arguments specify the starting and ending character numbers of\n\
 the region to operate on.  When used as a command, the text between\n\
 point and the mark is operated on.")
-  (b, e)
-     Lisp_Object b, e;
+  (beg, end)
+     Lisp_Object beg, end;
 {
-  casify_region (CASE_DOWN, b, e);
+  casify_region (CASE_DOWN, beg, end);
   return Qnil;
 }
 
@@ -169,20 +291,25 @@ Capitalized form means each word's first character is upper case\n\
 and the rest of it is lower case.\n\
 In programs, give two arguments, the starting and ending\n\
 character positions to operate on.")
-  (b, e)
-     Lisp_Object b, e;
+  (beg, end)
+     Lisp_Object beg, end;
 {
-  casify_region (CASE_CAPITALIZE, b, e);
+  casify_region (CASE_CAPITALIZE, beg, end);
   return Qnil;
 }
 
-/* Like Fcapitalize but change only the initials.  */
+/* Like Fcapitalize_region but change only the initials.  */
 
-Lisp_Object
-upcase_initials_region (b, e)
-     Lisp_Object b, e;
+DEFUN ("upcase-initials-region", Fupcase_initials_region,
+       Supcase_initials_region, 2, 2, "r",
+  "Upcase the initial of each word in the region.\n\
+Subsequent letters of each word are not changed.\n\
+In programs, give two arguments, the starting and ending\n\
+character positions to operate on.")
+  (beg, end)
+     Lisp_Object beg, end;
 {
-  casify_region (CASE_CAPITALIZE_UP, b, e);
+  casify_region (CASE_CAPITALIZE_UP, beg, end);
   return Qnil;
 }
 
@@ -193,14 +320,16 @@ operate_on_word (arg, newpoint)
 {
   Lisp_Object val;
   int farend;
+  int iarg;
 
   CHECK_NUMBER (arg, 0);
-  farend = scan_words (point, XINT (arg));
+  iarg = XINT (arg);
+  farend = scan_words (PT, iarg);
   if (!farend)
-    farend = XINT (arg) > 0 ? ZV : BEGV;
+    farend = iarg > 0 ? ZV : BEGV;
 
-  *newpoint = point > farend ? point : farend;
-  XFASTINT (val) = farend;
+  *newpoint = PT > farend ? PT : farend;
+  XSETFASTINT (val, farend);
 
   return val;
 }
@@ -214,7 +343,7 @@ See also `capitalize-word'.")
 {
   Lisp_Object beg, end;
   int newpoint;
-  XFASTINT (beg) = point;
+  XSETFASTINT (beg, PT);
   end = operate_on_word (arg, &newpoint);
   casify_region (CASE_UP, beg, end);
   SET_PT (newpoint);
@@ -229,7 +358,7 @@ With negative argument, convert previous words but do not move.")
 {
   Lisp_Object beg, end;
   int newpoint;
-  XFASTINT (beg) = point;
+  XSETFASTINT (beg, PT);
   end = operate_on_word (arg, &newpoint);
   casify_region (CASE_DOWN, beg, end);
   SET_PT (newpoint);
@@ -246,7 +375,7 @@ With negative argument, capitalize previous words but do not move.")
 {
   Lisp_Object beg, end;
   int newpoint;
-  XFASTINT (beg) = point;
+  XSETFASTINT (beg, PT);
   end = operate_on_word (arg, &newpoint);
   casify_region (CASE_CAPITALIZE, beg, end);
   SET_PT (newpoint);
@@ -255,12 +384,16 @@ With negative argument, capitalize previous words but do not move.")
 
 syms_of_casefiddle ()
 {
+  Qidentity = intern ("identity");
+  staticpro (&Qidentity);
   defsubr (&Supcase);
   defsubr (&Sdowncase);
   defsubr (&Scapitalize);
+  defsubr (&Supcase_initials);
   defsubr (&Supcase_region);
   defsubr (&Sdowncase_region);
   defsubr (&Scapitalize_region);
+  defsubr (&Supcase_initials_region);
   defsubr (&Supcase_word);
   defsubr (&Sdowncase_word);
   defsubr (&Scapitalize_word);
