@@ -17,14 +17,29 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 
 #include <config.h>
 #include <stdio.h>
 #include <ctype.h>
+
+#ifdef STDC_HEADERS
+#include <float.h>
+#endif
+
 #include "lisp.h"
 
+#ifndef DBL_MAX_10_EXP
+#define DBL_MAX_10_EXP 308 /* IEEE double */
+#endif
+
+/* Since we use the macro CHAR_HEAD_P, we have to include this, but
+   don't have to include others because CHAR_HEAD_P does not contains
+   another macro.  */
+#include "charset.h"
+
 extern long *xmalloc (), *xrealloc ();
 
 static int doprnt1 ();
@@ -79,10 +94,10 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
   register char *bufptr = buffer; /* Pointer into output buffer.. */
 
   /* Use this for sprintf unless we need something really big.  */
-  char tembuf[100];
+  char tembuf[DBL_MAX_10_EXP + 100];
 
   /* Size of sprintf_buffer.  */
-  int size_allocated = 100;
+  int size_allocated = sizeof (tembuf);
 
   /* Buffer to use for sprintf.  Either tembuf or same as BIG_BUFFER.  */
   char *sprintf_buffer = tembuf;
@@ -96,7 +111,7 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
   char *fmtcpy;
   int minlen;
   int size;			/* Field width factor; e.g., %90d */
-  char charbuf[2];		/* Used for %c.  */
+  char charbuf[5];		/* Used for %c.  */
 
   if (format_end == 0)
     format_end = format + strlen (format);
@@ -113,7 +128,8 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
     {
       if (*fmt == '%')	/* Check for a '%' character */
 	{
-	  int size_bound;
+	  int size_bound = 0;
+	  int width;		/* Columns occupied by STRING.  */
 
 	  fmt++;
 	  /* Copy this one %-spec into fmtcpy.  */
@@ -122,22 +138,38 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
 	  while (1)
 	    {
 	      *string++ = *fmt;
-	      if (! (*fmt >= '0' && *fmt <= '9')
-		  && *fmt != '-' && *fmt != ' '&& *fmt != '.')
+	      if ('0' <= *fmt && *fmt <= '9')
+		{
+		  /* Get an idea of how much space we might need.
+		     This might be a field width or a precision; e.g.
+		     %1.1000f and %1000.1f both might need 1000+ bytes.
+		     Parse the width or precision, checking for overflow.  */
+		  int n = *fmt - '0';
+		  while ('0' <= fmt[1] && fmt[1] <= '9')
+		    {
+		      if (n * 10 / 10 != n
+			  || (n = n * 10 + (fmt[1] - '0')) < 0)
+			error ("Format width or precision too large");
+		      *string++ = *++fmt;
+		    }
+
+		  if (size_bound < n)
+		    size_bound = n;
+		}
+	      else if (*fmt == '-' || *fmt == ' ' || *fmt == '.')
+		;
+	      else
 		break;
 	      fmt++;
 	    }
 	  *string = 0;
-	  /* Get an idea of how much space we might need.  */
-	  size_bound = atoi (&fmtcpy[1]);
 
-	  /* Avoid pitfall of negative "size" parameter ("%-200d"). */
+	  /* Make the size bound large enough to handle floating point formats
+	     with large numbers.  */
+	  size_bound += DBL_MAX_10_EXP + 50;
+
 	  if (size_bound < 0)
-	    size_bound = -size_bound;
-	  size_bound += 50;
-
-	  if (size_bound > (unsigned) (1 << (BITS_PER_INT - 1)))
-	    error ("Format padding too large");
+	    error ("Format width or precision too large");
 
 	  /* Make sure we have that much.  */
 	  if (size_bound > size_allocated)
@@ -200,8 +232,8 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
 		minlen = atoi (&fmtcpy[1]);
 	      if (lispstrings)
 		{
-		  string = (char *) XSTRING (((Lisp_Object *) args)[cnt])->data;
-		  tem = XSTRING (((Lisp_Object *) args)[cnt])->size;
+		  string = (char *) ((struct Lisp_String *)args[cnt])->data;
+		  tem = ((struct Lisp_String *)args[cnt])->size;
 		  cnt++;
 		}
 	      else
@@ -209,15 +241,21 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
 		  string = args[cnt++];
 		  tem = strlen (string);
 		}
+	      width = strwidth (string, tem);
 	      goto doit1;
 
 	      /* Copy string into final output, truncating if no room.  */
 	    doit:
-	      tem = strlen (string);
+	      /* Coming here means STRING contains ASCII only.  */
+	      width = tem = strlen (string);
 	    doit1:
+	      /* We have already calculated:
+		 TEM -- length of STRING,
+		 WIDTH -- columns occupied by STRING when displayed, and
+		 MINLEN -- minimum columns of the output.  */
 	      if (minlen > 0)
 		{
-		  while (minlen > tem && bufsize > 0)
+		  while (minlen > width && bufsize > 0)
 		    {
 		      *bufptr++ = ' ';
 		      bufsize--;
@@ -226,13 +264,21 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
 		  minlen = 0;
 		}
 	      if (tem > bufsize)
-		tem = bufsize;
-	      bcopy (string, bufptr, tem);
+		{
+		  /* Truncate the string at character boundary.  */
+		  tem = bufsize;
+		  while (!CHAR_HEAD_P (string + tem - 1)) tem--;
+		  bcopy (string, bufptr, tem);
+		  /* We must calculate WIDTH again.  */
+		  width = strwidth (bufptr, tem);
+		}
+	      else
+		bcopy (string, bufptr, tem);
 	      bufptr += tem;
 	      bufsize -= tem;
 	      if (minlen < 0)
 		{
-		  while (minlen < - tem && bufsize > 0)
+		  while (minlen < - width && bufsize > 0)
 		    {
 		      *bufptr++ = ' ';
 		      bufsize--;
@@ -245,9 +291,10 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
 	    case 'c':
 	      if (cnt == nargs)
 		error ("not enough arguments for format string");
-	      *charbuf = (EMACS_INT) args[cnt++];
-	      string = charbuf;
-	      tem = 1;
+	      tem = CHAR_STRING ((EMACS_INT) args[cnt], charbuf, string);
+	      cnt++;
+	      string[tem] = 0;
+	      width = strwidth (string, tem);
 	      if (fmtcpy[1] != 'c')
 		minlen = atoi (&fmtcpy[1]);
 	      goto doit1;
@@ -256,8 +303,20 @@ doprnt1 (lispstrings, buffer, bufsize, format, format_end, nargs, args)
 	      fmt--;    /* Drop thru and this % will be treated as normal */
 	    }
 	}
-      *bufptr++ = *fmt++;	/* Just some characters; Copy 'em */
-      bufsize--;
+
+      {
+	/* Just some character; Copy it if the whole multi-byte form
+	   fit in the buffer.  */
+	char *save_bufptr = bufptr;
+
+	do { *bufptr++ = *fmt++; }
+	while (--bufsize > 0 && !CHAR_HEAD_P (fmt));
+	if (!CHAR_HEAD_P (fmt))
+	  {
+	    bufptr = save_bufptr;
+	    break;
+	  }
+      }
     };
 
   /* If we had to malloc something, free it.  */
