@@ -103,6 +103,10 @@ Boston, MA 02111-1307, USA.  */
 /* We don't have -loldX, and we don't need it.  */
 #define LIB_XMENU_LIB
 
+/* doug@zadall.com says SCO 3.2v4.2 mktime botches time arithmetic as used
+   by display-time.  */
+#define BROKEN_MKTIME 1
+
 /* SCO does have TIOCGWINSZ.  */
 #undef BROKEN_TIOCGWINSZ
 #define NEED_PTEM_H
