@@ -49,7 +49,16 @@ extern sigset_t sys_sigmask ();
 #endif /* ! defined (__GNUC__) */
 #endif
 
+#ifndef sigpause
 #define sigpause(SIG)    sys_sigpause (SIG)
+#else
+/* If sigpause is predefined, with POSIX_SIGNALS,
+   let's assume it needs this kind of argument.
+   This is true for Glibc 2.1.  */
+#undef SIGEMPTYMASK
+#define SIGEMPTYMASK sigmask (0)
+#endif
+
 #define sigblock(SIG)    sys_sigblock (SIG)
 #define sigunblock(SIG)  sys_sigunblock (SIG)
 #ifndef sigsetmask
@@ -116,27 +125,18 @@ sigset_t sys_sigsetmask (/*sigset_t new_mask*/);
 { SIGMASKTYPE omask = sigblock (SIGFULLMASK); sigsetmask (omask & ~SIG); }
 #endif
 
-/* It would be very nice if we could somehow clean up all this trash.  */
-
 #ifndef BSD4_1
 #define sigfree() sigsetmask (SIGEMPTYMASK)
-#define sigholdx(sig) sigsetmask (sigmask (sig))
-#define sigblockx(sig) sigblock (sigmask (sig))
-#define sigunblockx(sig) sigunblock (sigmask (sig))
-#define sigpausex(sig) sigpause (0)
 #endif /* not BSD4_1 */
 
 #ifdef BSD4_1
 #define SIGIO SIGTINT
-/* sigfree and sigholdx are in sysdep.c */
-#define sigblockx(sig) sighold (sig)
-#define sigunblockx(sig) sigrelse (sig)
-#define sigpausex(sig) sigpause (sig)
+/* sigfree is in sysdep.c */
 #endif /* BSD4_1 */
 
 /* On bsd, [man says] kill does not accept a negative number to kill a pgrp.
    Must do that using the killpg call.  */
-#ifdef BSD
+#ifdef BSD_SYSTEM
 #define EMACS_KILLPG(gid, signo) (killpg ( (gid), (signo)))
 #else
 #ifdef WINDOWSNT
