@@ -59,7 +59,7 @@ Lisp_Object Qlocal_map;
 
 /* Visual properties text (including strings) may have.  */
 Lisp_Object Qforeground, Qbackground, Qfont, Qunderline, Qstipple;
-Lisp_Object Qinvisible, Qread_only, Qintangible, Qmouse_face;
+Lisp_Object Qinvisible, Qread_only, Qintangible;
 
 /* Sticky properties */
 Lisp_Object Qfront_sticky, Qrear_nonsticky;
@@ -150,9 +150,11 @@ validate_interval_range (object, begin, end, force)
       if (! (0 <= XINT (*begin) && XINT (*begin) <= XINT (*end)
 	     && XINT (*end) <= s->size))
 	args_out_of_range (*begin, *end);
-      XSETFASTINT (*begin, XFASTINT (*begin));
+      /* User-level Positions in strings start with 0,
+	 but the interval code always wants positions starting with 1.  */
+      XSETFASTINT (*begin, XFASTINT (*begin) + 1);
       if (begin != end)
-	XSETFASTINT (*end, XFASTINT (*end));
+	XSETFASTINT (*end, XFASTINT (*end) + 1);
       i = s->intervals;
 
       if (s->size == 0)
@@ -477,7 +479,7 @@ erase_properties (i)
 }
 #endif
 
-/* Returns the interval of POSITION in OBJECT. 
+/* Returns the interval of the POSITION in OBJECT. 
    POSITION is BEG-based.  */
 
 INTERVAL
@@ -490,8 +492,6 @@ interval_of (position, object)
 
   if (NILP (object))
     XSETBUFFER (object, current_buffer);
-  else if (EQ (object, Qt))
-    return NULL_INTERVAL;
 
   CHECK_STRING_OR_BUFFER (object, 0);
 
@@ -507,8 +507,9 @@ interval_of (position, object)
     {
       register struct Lisp_String *s = XSTRING (object);
 
-      beg = 0;
-      end = s->size;
+      /* We expect position to be 1-based.  */
+      beg = BEG;
+      end = s->size + BEG;
       i = s->intervals;
     }
 
@@ -522,9 +523,9 @@ interval_of (position, object)
 
 DEFUN ("text-properties-at", Ftext_properties_at,
        Stext_properties_at, 1, 2, 0,
-  "Return the list of properties of the character at POSITION in OBJECT.\n\
-OBJECT is the string or buffer to look for the properties in;\n\
-nil means the current buffer.\n\
+  "Return the list of properties held by the character at POSITION\n\
+in optional argument OBJECT, a string or buffer.  If nil, OBJECT\n\
+defaults to the current buffer.\n\
 If POSITION is at the end of OBJECT, the value is nil.")
   (position, object)
      Lisp_Object position, object;
@@ -713,7 +714,7 @@ past position LIMIT; return LIMIT if nothing is found before LIMIT.")
 				? XSTRING (object)->size
 				: BUF_ZV (XBUFFER (object))));
       else
-	XSETFASTINT (position, next->position);
+	XSETFASTINT (position, next->position - (STRINGP (object)));
       return position;
     }
 
@@ -731,7 +732,7 @@ past position LIMIT; return LIMIT if nothing is found before LIMIT.")
   if (! NILP (limit) && !(next->position < XFASTINT (limit)))
     return limit;
 
-  XSETFASTINT (position, next->position);
+  XSETFASTINT (position, next->position - (STRINGP (object)));
   return position;
 }
 
@@ -806,7 +807,8 @@ past position LIMIT; return LIMIT if nothing is found before LIMIT.")
   if (! NILP (limit) && !(next->position < XFASTINT (limit)))
     return limit;
 
-  return make_number (next->position);
+  XSETFASTINT (position, next->position - (STRINGP (object)));
+  return position;
 }
 
 DEFUN ("previous-property-change", Fprevious_property_change,
@@ -841,7 +843,7 @@ back past position LIMIT; return LIMIT if nothing is found until LIMIT.")
   previous = previous_interval (i);
   while (! NULL_INTERVAL_P (previous) && intervals_equal (previous, i)
 	 && (NILP (limit)
-	     || (previous->position + LENGTH (previous) > XFASTINT (limit))))
+	     || previous->position + LENGTH (previous) > XFASTINT (limit)))
     previous = previous_interval (previous);
   if (NULL_INTERVAL_P (previous))
     return limit;
@@ -849,7 +851,9 @@ back past position LIMIT; return LIMIT if nothing is found until LIMIT.")
       && !(previous->position + LENGTH (previous) > XFASTINT (limit)))
     return limit;
 
-  return make_number (previous->position + LENGTH (previous));
+  XSETFASTINT (position, (previous->position + LENGTH (previous)
+		     - (STRINGP (object))));
+  return position;
 }
 
 DEFUN ("previous-single-property-change", Fprevious_single_property_change,
@@ -889,7 +893,7 @@ back past position LIMIT; return LIMIT if nothing is found until LIMIT.")
   while (! NULL_INTERVAL_P (previous)
 	 && EQ (here_val, textget (previous->plist, prop))
 	 && (NILP (limit)
-	     || (previous->position + LENGTH (previous) > XFASTINT (limit))))
+	     || previous->position + LENGTH (previous) > XFASTINT (limit)))
     previous = previous_interval (previous);
   if (NULL_INTERVAL_P (previous))
     return limit;
@@ -897,7 +901,9 @@ back past position LIMIT; return LIMIT if nothing is found until LIMIT.")
       && !(previous->position + LENGTH (previous) > XFASTINT (limit)))
     return limit;
 
-  return make_number (previous->position + LENGTH (previous));
+  XSETFASTINT (position, (previous->position + LENGTH (previous)
+		     - (STRINGP (object))));
+  return position;
 }
 
 /* Callers note, this can GC when OBJECT is a buffer (or nil).  */
@@ -1279,7 +1285,7 @@ containing the text.")
 	  pos = i->position;
 	  if (pos < XINT (start))
 	    pos = XINT (start);
-	  return make_number (pos);
+	  return make_number (pos - (STRINGP (object)));
 	}
       i = next_interval (i);
     }
@@ -1315,13 +1321,121 @@ containing the text.")
 	{
 	  if (i->position > s)
 	    s = i->position;
-	  return make_number (s);
+	  return make_number (s - (STRINGP (object)));
 	}
       i = next_interval (i);
     }
   return Qnil;
 }
 
+#if 0 /* You can use set-text-properties for this.  */
+
+DEFUN ("erase-text-properties", Ferase_text_properties,
+       Serase_text_properties, 2, 3, 0,
+  "Remove all properties from the text from START to END.\n\
+The optional third argument, OBJECT,\n\
+is the string or buffer containing the text.")
+  (start, end, object)
+     Lisp_Object start, end, object;
+{
+  register INTERVAL i;
+  register INTERVAL prev_changed = NULL_INTERVAL;
+  register int s, len, modified;
+
+  if (NILP (object))
+    XSETBUFFER (object, current_buffer);
+
+  i = validate_interval_range (object, &start, &end, soft);
+  if (NULL_INTERVAL_P (i))
+    return Qnil;
+
+  s = XINT (start);
+  len = XINT (end) - s;
+
+  if (i->position != s)
+    {
+      register int got;
+      register INTERVAL unchanged = i;
+
+      /* If there are properties here, then this text will be modified.  */
+      if (! NILP (i->plist))
+	{
+	  i = split_interval_right (unchanged, s - unchanged->position);
+	  i->plist = Qnil;
+	  modified++;
+
+	  if (LENGTH (i) > len)
+	    {
+	      i = split_interval_right (i, len);
+	      copy_properties (unchanged, i);
+	      return Qt;
+	    }
+
+	  if (LENGTH (i) == len)
+	    return Qt;
+
+	  got = LENGTH (i);
+	}
+      /* If the text of I is without any properties, and contains
+         LEN or more characters, then we may return without changing
+	 anything.*/
+      else if (LENGTH (i) - (s - i->position) <= len)
+	return Qnil;
+      /* The amount of text to change extends past I, so just note
+	 how much we've gotten.  */
+      else
+	got = LENGTH (i) - (s - i->position);
+
+      len -= got;
+      prev_changed = i;
+      i = next_interval (i);
+    }
+
+  /* We are starting at the beginning of an interval, I.  */
+  while (len > 0)
+    {
+      if (LENGTH (i) >= len)
+	{
+	  /* If I has no properties, simply merge it if possible.  */
+	  if (NILP (i->plist))
+	    {
+	      if (! NULL_INTERVAL_P (prev_changed))
+		merge_interval_left (i);
+
+	      return modified ? Qt : Qnil;
+	    }
+
+          if (LENGTH (i) > len)
+            i = split_interval_left (i, len);
+	  if (! NULL_INTERVAL_P (prev_changed))
+	    merge_interval_left (i);
+	  else
+	    i->plist = Qnil;
+
+	  return Qt;
+	}
+
+      /* Here if we still need to erase past the end of I */
+      len -= LENGTH (i);
+      if (NULL_INTERVAL_P (prev_changed))
+	{
+	  modified += erase_properties (i);
+	  prev_changed = i;
+	}
+      else
+	{
+	  modified += ! NILP (i->plist);
+	  /* Merging I will give it the properties of PREV_CHANGED.  */
+	  prev_changed = i = merge_interval_left (i);
+	}
+
+      i = next_interval (i);
+    }
+
+  return modified ? Qt : Qnil;
+}
+#endif /* 0 */
+
 /* I don't think this is the right interface to export; how often do you
    want to do something like this, other than when you're copying objects
    around?
@@ -1337,7 +1451,7 @@ containing the text.")
    Return t if any property value actually changed, nil otherwise.  */
 
 /* Note this can GC when DEST is a buffer.  */
-
+
 Lisp_Object
 copy_text_properties (start, end, src, pos, dest, prop)
        Lisp_Object start, end, src, pos, dest, prop;
@@ -1439,13 +1553,12 @@ call_mod_hooks (list, start, end)
   UNGCPRO;
 }
 
-/* Check for read-only intervals between character positions START ... END,
-   in BUF, and signal an error if we find one.
-
-   Then check for any modification hooks in the range.
-   Create a list of all these hooks in lexicographic order,
-   eliminating consecutive extra copies of the same hook.  Then call
-   those hooks in order, with START and END - 1 as arguments.  */
+/* Check for read-only intervals and signal an error if we find one.
+   Then check for any modification hooks in the range START up to
+   (but not including) END.  Create a list of all these hooks in
+   lexicographic order, eliminating consecutive extra copies of the
+   same hook.  Then call those hooks in order, with START and END - 1
+   as arguments.  */
 
 void
 verify_interval_modification (buf, start, end)
@@ -1616,7 +1729,7 @@ verify_interval_modification (buf, start, end)
     }
 }
 
-/* Run the interval hooks for an insertion on character range START ... END.
+/* Run the interval hooks for an insertion.
    verify_interval_modification chose which hooks to run;
    this function is called after the insertion happens
    so it can indicate the range of inserted text.  */
@@ -1679,8 +1792,6 @@ This also inhibits the use of the `intangible' text property.");
   Qfront_sticky = intern ("front-sticky");
   staticpro (&Qrear_nonsticky);
   Qrear_nonsticky = intern ("rear-nonsticky");
-  staticpro (&Qmouse_face);
-  Qmouse_face = intern ("mouse-face");
 
   /* Properties that text might use to specify certain actions */
 
