@@ -428,43 +428,9 @@ Filesz      Memsz       Flags       Align
 #include <sys/elf_mips.h>
 #include <sym.h>
 #endif /* __sony_news && _SYSTYPE_SYSV */
-#if __sgi
-#include <sym.h> /* for HDRR declaration */
-#endif /* __sgi */
 
 #if defined (__alpha__) && !defined (__NetBSD__) && !defined (__OpenBSD__)
-/* Declare COFF debugging symbol table.  This used to be in
-   /usr/include/sym.h, but this file is no longer included in Red Hat
-   5.0 and presumably in any other glibc 2.x based distribution.  */
-typedef struct {
-	short magic;
-	short vstamp;
-	int ilineMax;
-	int idnMax;
-	int ipdMax;
-	int isymMax;
-	int ioptMax;
-	int iauxMax;
-	int issMax;
-	int issExtMax;
-	int ifdMax;
-	int crfd;
-	int iextMax;
-	long cbLine;
-	long cbLineOffset;
-	long cbDnOffset;
-	long cbPdOffset;
-	long cbSymOffset;
-	long cbOptOffset;
-	long cbAuxOffset;
-	long cbSsOffset;
-	long cbSsExtOffset;
-	long cbFdOffset;
-	long cbRfdOffset;
-	long cbExtOffset;
-} HDRR, *pHDRR; 
-#define cbHDRR sizeof(HDRR)
-#define hdrNil ((pHDRR)0)
+#include <sym.h>	/* get COFF debugging symbol table declaration */
 #endif
 
 #ifdef __NetBSD__
@@ -537,7 +503,7 @@ extern void fatal (char *, ...);
 /* Get the address of a particular section or program header entry,
  * accounting for the size of the entries.
  */
-/*
+/* 
    On PPC Reference Platform running Solaris 2.5.1
    the plt section is also of type NOBI like the bss section.
    (not really stored) and therefore sections after the bss
@@ -546,7 +512,7 @@ extern void fatal (char *, ...);
    Thus, we modify the test from
       if (NEW_SECTION_H (nn).sh_offset >= new_data2_offset)
    to
-      if (NEW_SECTION_H (nn).sh_offset >=
+      if (NEW_SECTION_H (nn).sh_offset >= 
                OLD_SECTION_H (old_bss_index-1).sh_offset)
    This is just a hack. We should put the new data section
    before the .plt section.
@@ -578,9 +544,9 @@ typedef unsigned char byte;
 
 /* Round X up to a multiple of Y.  */
 
-static ElfW(Addr)
+int
 round_up (x, y)
-     ElfW(Addr) x, y;
+     int x, y;
 {
   int rem = x % y;
   if (rem == 0)
@@ -623,7 +589,9 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
   ElfW(Addr) new_data2_addr;
 
   int n, nn, old_bss_index, old_data_index, new_data2_index;
+#if defined ( __sony_news) && defined (_SYSTYPE_SYSV)
   int old_sbss_index, old_mdebug_index;
+#endif /* __sony_news && _SYSTYPE_SYSV */
   struct stat stat_buf;
 
   /* Open the old file & map it into the address space. */
@@ -673,6 +641,7 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
   if (old_bss_index == old_file_h->e_shnum)
     fatal ("Can't find .bss in %s.\n", old_name, 0);
 
+#if defined (__sony_news) && defined (_SYSTYPE_SYSV)
   for (old_sbss_index = 1; old_sbss_index < (int) old_file_h->e_shnum;
        old_sbss_index++)
     {
@@ -686,7 +655,6 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
     }
   if (old_sbss_index == old_file_h->e_shnum)
     {
-      old_sbss_index = -1;
       old_bss_addr = OLD_SECTION_H(old_bss_index).sh_addr;
       old_bss_size = OLD_SECTION_H(old_bss_index).sh_size;
       new_data2_offset = OLD_SECTION_H(old_bss_index).sh_offset;
@@ -714,7 +682,10 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
     }
     if (old_mdebug_index == old_file_h->e_shnum)
 	old_mdebug_index = 0;
-
+#else /* not (__sony_news && _SYSTYPE_SYSV) */	    
+  old_bss_addr = OLD_SECTION_H (old_bss_index).sh_addr;
+  old_bss_size = OLD_SECTION_H (old_bss_index).sh_size;
+#endif /* not (__sony_news && _SYSTYPE_SYSV) */	    
 #if defined (emacs) || !defined (DEBUG)
   new_bss_addr = (ElfW(Addr)) sbrk (0);
 #else
@@ -722,6 +693,9 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 #endif
   new_data2_addr = old_bss_addr;
   new_data2_size = new_bss_addr - old_bss_addr;
+#if !defined (__sony_news) || !defined (_SYSTYPE_SYSV)
+  new_data2_offset = OLD_SECTION_H (old_bss_index).sh_offset;
+#endif /*  not (__sony_news && _SYSTYPE_SYSV) */
 
 #ifdef DEBUG
   fprintf (stderr, "old_bss_index %d\n", old_bss_index);
@@ -802,22 +776,18 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
   for (n = new_file_h->e_phnum - 1; n >= 0; n--)
     {
       /* Compute maximum of all requirements for alignment of section.  */
-      ElfW(Word) alignment = (NEW_PROGRAM_H (n)).p_align;
+      int alignment = (NEW_PROGRAM_H (n)).p_align;
       if ((OLD_SECTION_H (old_bss_index)).sh_addralign > alignment)
 	alignment = OLD_SECTION_H (old_bss_index).sh_addralign;
 
-#ifdef __mips
-	  /* According to r02kar@x4u2.desy.de (Karsten Kuenne)
-	     and oliva@gnu.org (Alexandre Oliva), on IRIX 5.2, we
-	     always get "Program segment above .bss" when dumping
-	     when the executable doesn't have an sbss section.  */
-      if (old_sbss_index != -1)
-#endif /* __mips */
+#if defined (__sony_news) && defined (_SYSTYPE_SYSV)
       if (NEW_PROGRAM_H (n).p_vaddr + NEW_PROGRAM_H (n).p_filesz
-	  > (old_sbss_index == -1
-	     ? old_bss_addr
-	     : round_up (old_bss_addr, alignment)))
-	  fatal ("Program segment above .bss in %s\n", old_name, 0);
+	  > round_up (old_bss_addr, alignment))
+	fatal ("Program segment above .bss in %s\n", old_name, 0);
+#else /* not (__sony_news && _SYSTYPE_SYSV) */
+      if (NEW_PROGRAM_H (n).p_vaddr + NEW_PROGRAM_H (n).p_filesz > old_bss_addr)
+	fatal ("Program segment above .bss in %s\n", old_name, 0);
+#endif /* not (__sony_news && _SYSTYPE_SYSV) */
 
       if (NEW_PROGRAM_H (n).p_type == PT_LOAD
 	  && (round_up ((NEW_PROGRAM_H (n)).p_vaddr
@@ -829,9 +799,7 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
   if (n < 0)
     fatal ("Couldn't find segment next to .bss in %s\n", old_name, 0);
 
-  /* Make sure that the size includes any padding before the old .bss
-     section.  */
-  NEW_PROGRAM_H (n).p_filesz = new_bss_addr - NEW_PROGRAM_H (n).p_vaddr;
+  NEW_PROGRAM_H (n).p_filesz += new_data2_size;
   NEW_PROGRAM_H (n).p_memsz = NEW_PROGRAM_H (n).p_filesz;
 
 #if 0 /* Maybe allow section after data2 - does this ever happen? */
@@ -865,10 +833,17 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
   for (n = 1, nn = 1; n < (int) old_file_h->e_shnum; n++, nn++)
     {
       caddr_t src;
+      int temp_index;
+#if defined (__sony_news) && defined (_SYSTYPE_SYSV)
       /* If it is (s)bss section, insert the new data2 section before it.  */
       /* new_data2_index is the index of either old_sbss or old_bss, that was
 	 chosen as a section for new_data2.   */
-      if (n == new_data2_index)
+      temp_index = new_data2_index;
+#else /* not (__sony_news && _SYSTYPE_SYSV) */
+      /* If it is bss section, insert the new data2 section before it.  */
+      temp_index = old_bss_index;
+#endif /* not (__sony_news && _SYSTYPE_SYSV) */
+      if (n == temp_index)
 	{
 	  /* Steal the data section header for this data2 section. */
 	  memcpy (&NEW_SECTION_H (nn), &OLD_SECTION_H (old_data_index),
@@ -893,16 +868,16 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 	      old_file_h->e_shentsize);
       
       if (n == old_bss_index
+#if defined (__sony_news) && defined (_SYSTYPE_SYSV)
 	  /* The new bss and sbss section's size is zero, and its file offset
 	     and virtual address should be off by NEW_DATA2_SIZE.  */
 	  || n == old_sbss_index
+#endif /* __sony_news and _SYSTYPE_SYSV */
 	  )
 	{
-	  /* NN should be `old_s?bss_index + 1' at this point. */
-	  NEW_SECTION_H (nn).sh_offset =
-	    NEW_SECTION_H (new_data2_index).sh_offset + new_data2_size;
-	  NEW_SECTION_H (nn).sh_addr =
-	    NEW_SECTION_H (new_data2_index).sh_addr + new_data2_size;
+	  /* NN should be `old_bss_index + 1' at this point. */
+	  NEW_SECTION_H (nn).sh_offset += new_data2_size;
+	  NEW_SECTION_H (nn).sh_addr += new_data2_size;
 	  /* Let the new bss section address alignment be the same as the
 	     section address alignment followed the old bss section, so
 	     this section will be placed in exactly the same place. */
@@ -958,16 +933,18 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 	 ".data" in the strings table) get copied from the current process
 	 instead of the old file.  */
       if (!strcmp (old_section_names + NEW_SECTION_H (n).sh_name, ".data")
+#ifdef _nec_ews_svr4				/* hir, 1994.6.13 */
+	  || !strcmp ((old_section_names + NEW_SECTION_H(n).sh_name),
+		      ".sdata")
+#endif
+#if defined (__sony_news) && defined (_SYSTYPE_SYSV)
 	  || !strcmp ((old_section_names + NEW_SECTION_H (n).sh_name),
 		      ".sdata")
-#ifdef __sgi
 	  || !strcmp ((old_section_names + NEW_SECTION_H (n).sh_name),
 		      ".lit4")
 	  || !strcmp ((old_section_names + NEW_SECTION_H (n).sh_name),
 		      ".lit8")
-#endif /* __sgi */
-	  || !strcmp ((old_section_names + NEW_SECTION_H (n).sh_name),
-		      ".sdata1")
+#endif /* __sony_news && _SYSTYPE_SYSV */
 	  || !strcmp ((old_section_names + NEW_SECTION_H (n).sh_name),
 		      ".data1"))
 	src = (caddr_t) OLD_SECTION_H (n).sh_addr;
@@ -1021,62 +998,6 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 	    }
 	}
 #endif /* __sony_news && _SYSTYPE_SYSV */
-
-#if __sgi
-      /* Adjust  the HDRR offsets in .mdebug and copy the 
-	 line data if it's in its usual 'hole' in the object.
-	 Makes the new file debuggable with dbx.
-	 patches up two problems: the absolute file offsets
-	 in the HDRR record of .mdebug (see /usr/include/syms.h), and
-	 the ld bug that gets the line table in a hole in the
-	 elf file rather than in the .mdebug section proper.
-	 David Anderson. davea@sgi.com  Jan 16,1994.  */
-      if (n == old_mdebug_index)
-	{
-#define MDEBUGADJUST(__ct,__fileaddr)		\
-  if (n_phdrr->__ct > 0)			\
-    {						\
-      n_phdrr->__fileaddr += movement;		\
-    }
-
-	  HDRR * o_phdrr = (HDRR *)((byte *)old_base + OLD_SECTION_H (n).sh_offset);
-	  HDRR * n_phdrr = (HDRR *)((byte *)new_base + NEW_SECTION_H (nn).sh_offset);
-	  unsigned movement = new_data2_size;
-
-	  MDEBUGADJUST (idnMax, cbDnOffset);
-	  MDEBUGADJUST (ipdMax, cbPdOffset);
-	  MDEBUGADJUST (isymMax, cbSymOffset);
-	  MDEBUGADJUST (ioptMax, cbOptOffset);
-	  MDEBUGADJUST (iauxMax, cbAuxOffset);
-	  MDEBUGADJUST (issMax, cbSsOffset);
-	  MDEBUGADJUST (issExtMax, cbSsExtOffset);
-	  MDEBUGADJUST (ifdMax, cbFdOffset);
-	  MDEBUGADJUST (crfd, cbRfdOffset);
-	  MDEBUGADJUST (iextMax, cbExtOffset);
-	  /* The Line Section, being possible off in a hole of the object,
-	     requires special handling.  */
-	  if (n_phdrr->cbLine > 0)
-	    {
-	      if (o_phdrr->cbLineOffset > (OLD_SECTION_H (n).sh_offset
-					   + OLD_SECTION_H (n).sh_size))
-		{
-		  /* line data is in a hole in elf. do special copy and adjust
-		     for this ld mistake.
-		     */
-		  n_phdrr->cbLineOffset += movement;
-
-		  memcpy (n_phdrr->cbLineOffset + new_base,
-			  o_phdrr->cbLineOffset + old_base, n_phdrr->cbLine);
-		}
-	      else
-		{
-		  /* somehow line data is in .mdebug as it is supposed to be.  */
-		  MDEBUGADJUST (cbLine, cbLineOffset);
-		}
-	    }
-	}
-#endif /* __sgi */
-
       /* If it is the symbol table, its st_shndx field needs to be patched.  */
       if (NEW_SECTION_H (nn).sh_type == SHT_SYMTAB
 	  || NEW_SECTION_H (nn).sh_type == SHT_DYNSYM)
@@ -1114,9 +1035,7 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 
       for (; symp < symendp; symp ++)
 	if (strcmp ((char *) (symnames + symp->st_name), "_end") == 0
-	    || strcmp ((char *) (symnames + symp->st_name), "end") == 0
-	    || strcmp ((char *) (symnames + symp->st_name), "_edata") == 0
-	    || strcmp ((char *) (symnames + symp->st_name), "edata") == 0)
+	    || strcmp ((char *) (symnames + symp->st_name), "_edata") == 0)
 	  memcpy (&symp->st_value, &new_bss_addr, sizeof (new_bss_addr));
     }
 
@@ -1135,16 +1054,6 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 	   member.  */
 	nn = section.sh_info;
 	if (!strcmp (old_section_names + NEW_SECTION_H (nn).sh_name, ".data")
-	    || !strcmp ((old_section_names + NEW_SECTION_H (nn).sh_name),
-			".sdata")
-#ifdef __sgi
-	    || !strcmp ((old_section_names + NEW_SECTION_H (nn).sh_name),
-			".lit4")
-	    || !strcmp ((old_section_names + NEW_SECTION_H (nn).sh_name),
-			".lit8")
-#endif /* __sgi */
-	    || !strcmp ((old_section_names + NEW_SECTION_H (nn).sh_name),
-			".sdata1")
 	    || !strcmp ((old_section_names + NEW_SECTION_H (nn).sh_name),
 			".data1"))
 	  {
